#include <clutter/clutter.h>
#include <mx/mx.h>

static gboolean
close_cb(ClutterStage* stage, ClutterEvent* event, gpointer data)
{
    clutter_main_quit();
    return TRUE;
}

int main(int argc, char** argv)
{
    if (clutter_init(&argc, &argv) != CLUTTER_INIT_SUCCESS) {
        g_error("Failed to initialize clutter.");
        return 1;
    }

    MxWindow* window = mx_window_new();
    ClutterStage* stage = mx_window_get_clutter_stage(window);
    clutter_actor_set_size(CLUTTER_ACTOR(stage), 500, 100);

    g_signal_connect(window, "destroy",
                     G_CALLBACK(close_cb), NULL);

    MxToolbar* toolbar = mx_window_get_toolbar(window);

    ClutterActor* box = mx_box_layout_new();
    mx_box_layout_set_orientation(MX_BOX_LAYOUT(box), MX_ORIENTATION_VERTICAL);

    mx_window_set_child(window, box);

    ClutterActor* scroll_view = mx_scroll_view_new();
    clutter_actor_set_height(scroll_view, 300);
    clutter_container_add_actor(CLUTTER_CONTAINER(box), scroll_view);

    ClutterActor* viewport = mx_viewport_new();
    clutter_container_add_actor(CLUTTER_CONTAINER(scroll_view), viewport);

    ClutterActor* label = mx_label_new();
    clutter_container_add_actor(CLUTTER_CONTAINER(viewport), label);

    mx_label_set_line_wrap(MX_LABEL(label), TRUE);

    char* filename = g_build_filename(g_get_home_dir(), ".bash_history", NULL);

    char* contents;
    gsize length;
    GError* error = NULL;
    g_file_get_contents(filename, &contents, &length, &error);

    if (error) {
        mx_label_set_text(MX_LABEL(label), error->message);
        g_clear_error(&error);
    } else {
        mx_label_set_text(MX_LABEL(label), contents);
        g_free(contents);
    }

    mx_window_show(window);

    clutter_main();

    return 0;
}
