#!/bin/bash
# vim: set sts=4 sw=4 et tw=0 :

set -e

TESTDIR=$(cd $(dirname $0); pwd; cd - &>/dev/null)
. "${TESTDIR}/config.sh"

V4L_NR="0"
V4L_DEVICE="/dev/video${V4L_NR}"
V4L_TEST="v4l_test"

#########
# Setup #
#########
trap "setup_failure" ERR

# chaiwala-v4l2fb executable
if [[ -n ${V4L2FB} ]]; then
    :
elif type -P chaiwala-v4l2fb >/dev/null; then
    V4L2FB="$(type -P chaiwala-v4l2fb)"
else
    cry "Unable to find chaiwala-v4l2fb!"
    setup_failure
fi

if ! arch_is_arm; then
    sudo modprobe -r vivid
    sudo modprobe vivid
fi

setup_success

###########
# Execute #
###########
test_vivid_to_stdout() {
    local outfile expected_size actual_size

    outfile="${WORKDIR}/v4l_out.bin"
    expected_size=$((10*640*360*32/8))

    # Get 10 frames from vivid
    "${V4L2FB}" "${V4L_DEVICE}" "${V4L_TEST}" 10 > "${outfile}"
    sync
    actual_size=$(wc -c < "${outfile}")

    if [[ $expected_size != $actual_size ]]; then
        echo "Expected size ($expected_size) != actual size ($actual_size)!"
        return 1
    fi

    # Remove only if exiting successfully
    rm -f "${outfile}"
}

trap "test_failure" ERR

if arch_is_arm; then
    say "Test only works on x86, skipping..."
    src_test_pass <<-EOF
#test_vivid_to_stdout
EOF
else
    src_test_pass <<-EOF
test_vivid_to_stdout
EOF
fi

test_success
